import {IS_PRODUCTION} from '../src/constants';


export function getStorageKey(key, defaultValue = []) {
  return new Promise((resolve, reject) => {
    try {
      chrome.storage.local.get({[key]: defaultValue}, (res) => {
        resolve(res[key]);
      });
    } catch (e) {
      reject(e);
    }
  });
}

export function setStorageKey(key, data) {
  return new Promise((resolve, reject) => {
    try {
      chrome.storage.local.set({[key]: data});
      resolve();
    } catch (e) {
      reject(e);
    }
  });
}

export function updateMemo(id, data) {
  return getStorageKey('items')
      .then((items) => {
        return items.map((item) => {
          if (item.id === id) {
            return {...item, ...data};
          }
          return item;
        });
      })
      .then((updatedMemos) => {
        return setStorageKey('items', updatedMemos);
      });
}

export function createMemo(data) {
  return getStorageKey('items')
      .then((items) => [...items, {...data, id: generateId()}])
      .then((updatedMemos) => {
        return setStorageKey('items', updatedMemos);
      });
}


function generateId() {
  return new Date().getTime() + new Date().getMilliseconds();
}

async function addMessage(text) {
  let messages = await getStorageKey('messages');

  if (!messages) messages = [];

  // TODO: generate reliable id

  const id = generateId();

  messages.push({id, text});

  setStorageKey('messages', messages)
      .catch((e) => {
        console.error(e);
      });
}

async function getMessages(id = null) {
  // all by default
  let messages = await getStorageKey('messages');
  if (id) {
    messages = messages.filter((message) => message.id === id);
  }
  return messages;
}

const Storage = {
  messages: {
    add: addMessage,
    get: getMessages,
  },
};

/*
* Debug
*/

// TODO: remove on prod
// debug
window.resetStorage = () => {
  setStorageKey('messages', null);
};


class StorageObserver {
  constructor() {
    this.subscribers = [];
    const that = this;

    chrome.storage.onChanged.addListener(function(changes, namespace) {
      for (const key in changes) {
        const storageChange = changes[key];
        if (!IS_PRODUCTION) {
          console.log(
              'Storage key "%s" in namespace "%s" changed. ',
              'Old value was "%s", new value is "%s".',
              key,
              namespace,
              storageChange.oldValue,
              storageChange.newValue,
          );
        }
        if (that.subscribers.length) {
          that.subscribers.forEach((cb) => {
            cb(key,
                namespace,
                storageChange.oldValue,
                storageChange.newValue);
          });
        }
      }
    });
  }

  subscribe(cb) {
    this.subscribers.push(cb);
  }

  unsubscribe(cb) {
    this.subscribers.filter((func) => func !== cb);
  }
}

const storageObserver = new StorageObserver();
Object.freeze(storageObserver);

export default storageObserver;
